@@ -91,8 +91,8 @@ use super::FileLock;
 use crate::CargoResult;
 use crate::GlobalContext;
 use anyhow::Context as _;
-use std::cell::RefCell;
 use std::io;
+use std::sync::Mutex;
 
 /// The style of lock to acquire.
 #[derive(Copy, Clone, Debug, PartialEq)]
@@ -435,7 +435,11 @@ pub struct CacheLock<'lock> {
 impl Drop for CacheLock<'_> {
     fn drop(&mut self) {
         use CacheLockMode::*;
-        let mut state = self.locker.state.borrow_mut();
+        let mut state = match self.locker.state.lock() {
+            Ok(result) => result,
+            // we should release the cache even if a thread panicked while holding a lock
+            Err(poison) => poison.into_inner(),
+        };
         match self.mode {
             Shared => {
                 state.mutate_lock.decrement();
@@ -472,24 +476,25 @@ pub struct CacheLocker {
     ///
     /// [`CacheLocker`] uses interior mutability because it is stuffed inside
     /// [`GlobalContext`], which does not allow mutation.
-    state: RefCell<CacheState>,
+    state: Mutex<CacheState>,
 }
 
 impl CacheLocker {
     /// Creates a new `CacheLocker`.
     pub fn new() -> CacheLocker {
         CacheLocker {
-            state: RefCell::new(CacheState {
+            state: CacheState {
                 cache_lock: RecursiveLock::new(CACHE_LOCK_NAME),
                 mutate_lock: RecursiveLock::new(MUTATE_NAME),
-            }),
+            }
+            .into(),
         }
     }
 
     /// Acquires a lock with the given mode, possibly blocking if another
     /// cargo is holding the lock.
     pub fn lock(&self, gctx: &GlobalContext, mode: CacheLockMode) -> CargoResult<CacheLock<'_>> {
-        let mut state = self.state.borrow_mut();
+        let mut state = self.state.lock().unwrap();
         let _ = state.lock(gctx, mode, Blocking)?;
         Ok(CacheLock { mode, locker: self })
     }
@@ -501,7 +506,7 @@ impl CacheLocker {
         gctx: &GlobalContext,
         mode: CacheLockMode,
     ) -> CargoResult<Option<CacheLock<'_>>> {
-        let mut state = self.state.borrow_mut();
+        let mut state = self.state.lock().unwrap();
         if state.lock(gctx, mode, NonBlocking)? == LockAcquired {
             Ok(Some(CacheLock { mode, locker: self }))
         } else {
@@ -519,7 +524,7 @@ impl CacheLocker {
     /// `DownloadExclusive` will return true if a `MutateExclusive` lock is
     /// held since they overlap.
     pub fn is_locked(&self, mode: CacheLockMode) -> bool {
-        let state = self.state.borrow();
+        let state = self.state.lock().unwrap();
         match (
             mode,
             state.cache_lock.count,
