@@ -549,86 +549,7 @@ fn build_script_with_conflicting_out_dirs() {
                 build = ["build1.rs", "build2.rs"]
             "#,
         )
-        // OUT_DIR is set to the lexicographically largest build script's OUT_DIR by default
-        .file(
-            "src/main.rs",
-            r#"
-                include!(concat!(env!("OUT_DIR"), "/foo.rs"));
-                fn main() {
-                    println!("{}", message());
-                }
-            "#,
-        )
-        .file(
-            "build1.rs",
-            r#"
-            use std::env;
-            use std::fs;
-            use std::path::Path;
-
-            fn main() {
-                let out_dir = env::var_os("OUT_DIR").unwrap();
-                let dest_path = Path::new(&out_dir).join("foo.rs");
-                fs::write(
-                    &dest_path,
-                    "pub fn message() -> &'static str {
-                        \"Hello, from Build Script 1!\"
-                    }
-                    "
-                ).unwrap();
-             }"#,
-        )
-        .file(
-            "build2.rs",
-            r#"
-            use std::env;
-            use std::fs;
-            use std::path::Path;
-
-            fn main() {
-                let out_dir = env::var_os("OUT_DIR").unwrap();
-                let dest_path = Path::new(&out_dir).join("foo.rs");
-                fs::write(
-                    &dest_path,
-                    "pub fn message() -> &'static str {
-                        \"Hello, from Build Script 2!\"
-                    }
-                    "
-                ).unwrap();
-             }"#,
-        )
-        .build();
-
-    p.cargo("run -v")
-        .masquerade_as_nightly_cargo(&["multiple-build-scripts"])
-        .with_status(0)
-        .with_stdout_data(str![[r#"
-Hello, from Build Script 2!
-
-"#]])
-        .run();
-}
-
-#[cargo_test]
-fn build_script_with_conflicts_reverse_sorted() {
-    // In this, multiple scripts create file with same name in their respective OUT_DIR.
-    // It is different from above because `package.build` is not sorted in this.
-
-    let p = project()
-        .file(
-            "Cargo.toml",
-            r#"
-                cargo-features = ["multiple-build-scripts"]
-
-                [package]
-                name = "foo"
-                version = "0.1.0"
-                edition = "2024"
-
-                build = ["build2.rs", "build1.rs"]
-            "#,
-        )
-        // OUT_DIR is set to the lexicographically largest build script's OUT_DIR by default
+        // By default, OUT_DIR is set to that of the first build script in the array
         .file(
             "src/main.rs",
             r#"
@@ -688,6 +609,85 @@ Hello, from Build Script 1!
         .run();
 }
 
+#[cargo_test]
+fn build_script_with_conflicts_reverse_sorted() {
+    // In this, multiple scripts create file with same name in their respective OUT_DIR.
+    // It is different from above because `package.build` is not sorted in this.
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                cargo-features = ["multiple-build-scripts"]
+
+                [package]
+                name = "foo"
+                version = "0.1.0"
+                edition = "2024"
+
+                build = ["build2.rs", "build1.rs"]
+            "#,
+        )
+        // By default, OUT_DIR is set to that of the first build script in the array
+        .file(
+            "src/main.rs",
+            r#"
+                include!(concat!(env!("OUT_DIR"), "/foo.rs"));
+                fn main() {
+                    println!("{}", message());
+                }
+            "#,
+        )
+        .file(
+            "build1.rs",
+            r#"
+            use std::env;
+            use std::fs;
+            use std::path::Path;
+
+            fn main() {
+                let out_dir = env::var_os("OUT_DIR").unwrap();
+                let dest_path = Path::new(&out_dir).join("foo.rs");
+                fs::write(
+                    &dest_path,
+                    "pub fn message() -> &'static str {
+                        \"Hello, from Build Script 1!\"
+                    }
+                    "
+                ).unwrap();
+             }"#,
+        )
+        .file(
+            "build2.rs",
+            r#"
+            use std::env;
+            use std::fs;
+            use std::path::Path;
+
+            fn main() {
+                let out_dir = env::var_os("OUT_DIR").unwrap();
+                let dest_path = Path::new(&out_dir).join("foo.rs");
+                fs::write(
+                    &dest_path,
+                    "pub fn message() -> &'static str {
+                        \"Hello, from Build Script 2!\"
+                    }
+                    "
+                ).unwrap();
+             }"#,
+        )
+        .build();
+
+    p.cargo("run -v")
+        .masquerade_as_nightly_cargo(&["multiple-build-scripts"])
+        .with_status(0)
+        .with_stdout_data(str![[r#"
+Hello, from Build Script 2!
+
+"#]])
+        .run();
+}
+
 #[cargo_test]
 fn rerun_untracks_other_files() {
     let p = project()
@@ -764,3 +764,118 @@ fn bar() {
 "#]])
         .run();
 }
+
+#[cargo_test]
+fn multiple_out_dirs() {
+    // Test to verify access to the `OUT_DIR` of the respective build scripts.
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                cargo-features = ["multiple-build-scripts"]
+
+                [package]
+                name = "foo"
+                version = "0.1.0"
+                edition = "2024"
+                build = ["build1.rs", "build2.rs"]
+            "#,
+        )
+        .file(
+            "src/main.rs",
+            r#"
+                include!(concat!(env!("build1_OUT_DIR"), "/foo.rs"));
+                include!(concat!(env!("build2_OUT_DIR"), "/foo.rs"));
+                fn main() {
+                    println!("{}", message1());
+                    println!("{}", message2());
+                }
+            "#,
+        )
+        .file(
+            "build1.rs",
+            r#"
+            use std::env;
+            use std::fs;
+            use std::path::Path;
+
+            fn main() {
+                let out_dir = env::var_os("OUT_DIR").unwrap();
+                let dest_path = Path::new(&out_dir).join("foo.rs");
+                fs::write(
+                    &dest_path,
+                    "pub fn message1() -> &'static str {
+                        \"Hello, from Build Script 1!\"
+                    }
+                    "
+                ).unwrap();
+             }"#,
+        )
+        .file(
+            "build2.rs",
+            r#"
+            use std::env;
+            use std::fs;
+            use std::path::Path;
+
+            fn main() {
+                let out_dir = env::var_os("OUT_DIR").unwrap();
+                let dest_path = Path::new(&out_dir).join("foo.rs");
+                fs::write(
+                    &dest_path,
+                    "pub fn message2() -> &'static str {
+                        \"Hello, from Build Script 2!\"
+                    }
+                    "
+                ).unwrap();
+             }"#,
+        )
+        .build();
+
+    p.cargo("run -v")
+        .masquerade_as_nightly_cargo(&["multiple-build-scripts"])
+        .with_status(0)
+        .with_stdout_data(str![[r#"
+Hello, from Build Script 1!
+Hello, from Build Script 2!
+
+"#]])
+        .run();
+}
+
+#[cargo_test]
+fn duplicate_build_script_stems() {
+    // Test to verify that duplicate build script file stems throws error.
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                cargo-features = ["multiple-build-scripts"]
+
+                [package]
+                name = "foo"
+                version = "0.1.0"
+                edition = "2024"
+                build = ["build1.rs", "foo/build1.rs"]
+            "#,
+        )
+        .file("src/main.rs", "fn main() {}")
+        .file("build1.rs", "fn main() {}")
+        .file("foo/build1.rs", "fn main() {}")
+        .build();
+
+    p.cargo("check -v")
+        .masquerade_as_nightly_cargo(&["multiple-build-scripts"])
+        .with_status(101)
+        .with_stderr_data(str![[r#"
+[ERROR] failed to parse manifest at `[ROOT]/foo/Cargo.toml`
+
+Caused by:
+  found build scripts with duplicate file stems, but all build scripts must have a unique file stem
+    for stem `build1`: build1.rs, foo/build1.rs
+
+"#]])
+        .run();
+}
